package yaCloud

import (
	"bytes"
	"crypto/rsa"
	"encoding/json"
	"fmt"
	"io"
	"log"
	"net/http"
	"net/url"
	"os"
	"strings"
	"time"

	"github.com/golang-jwt/jwt/v4"
)

var (
	keyID            = ""
	serviceAccountID = ""
	keyFile          = "private.key"
	endpoint         = "https://storage.api.cloud.yandex.net/storage/v1/buckets"
)

func init() {
	keyID = os.Getenv("YA_KEY_ID")
	serviceAccountID = os.Getenv("YA_SERVICE_ACC_ID")
	if val, ok := os.LookupEnv("YA_KEY_FILE"); ok {
		keyFile = val
	}
}

type Client struct {
	client *http.Client
	bucket string
}

type transport struct {
	iam                 string
	underlyingTransport http.RoundTripper
}

func (t *transport) RoundTrip(req *http.Request) (*http.Response, error) {
	req.Header.Set("Authorization", fmt.Sprintf("Bearer %v", t.iam))
	req.Header.Set("Content-Type", "application/json")
	return t.underlyingTransport.RoundTrip(req)
}

func NewClient(bucket string) *Client {
	client := http.DefaultClient
	client.Transport = &transport{underlyingTransport: http.DefaultTransport, iam: getIAMToken()}

	return &Client{
		client: http.DefaultClient,
		bucket: bucket,
	}
}

func (c *Client) GetBucket() (*Bucket, error) {
	data := url.Values{}
	data.Set("view", "VIEW_FULL")
	href := fmt.Sprintf("%v/%v?%v", endpoint, c.bucket, data.Encode())

	req, err := http.NewRequest("GET", href, nil)

	if err != nil {
		return nil, err
	}

	resp, err := c.client.Do(req)

	if err != nil {
		return nil, err
	}
	defer resp.Body.Close()

	if err = checkError(resp); err != nil {
		return nil, err
	}

	var r Bucket
	if err = json.NewDecoder(resp.Body).Decode(&r); err != nil {
		return nil, err
	}

	return &r, err
}

func (c *Client) GetBucketStats() (*BucketStats, error) {
	href := fmt.Sprintf("%v/%v:getStats", endpoint, c.bucket)

	req, err := http.NewRequest("GET", href, nil)

	if err != nil {
		return nil, err
	}

	resp, err := c.client.Do(req)

	if err != nil {
		return nil, err
	}
	defer resp.Body.Close()

	if err = checkError(resp); err != nil {
		return nil, err
	}

	var r BucketStats
	if err = json.NewDecoder(resp.Body).Decode(&r); err != nil {
		return nil, err
	}

	return &r, err
}

func (c *Client) UpdateBucketMaxSize(quota uint64) error {
	href := fmt.Sprintf("%v/%v", endpoint, c.bucket)

	body, err := json.Marshal(NewReqUpdateBucketMaxSize(quota))
	if err != nil {
		return err
	}

	req, err := http.NewRequest("PATCH", href, bytes.NewBuffer(body))
	if err != nil {
		log.Fatal(err)
	}

	resp, err := c.client.Do(req)

	if err != nil {
		return err
	}
	defer resp.Body.Close()

	defer resp.Body.Close()

	if err = checkError(resp); err != nil {
		return err
	}

	var r RespUpdateBucketMaxSize

	if err = json.NewDecoder(resp.Body).Decode(&r); err != nil {
		return err
	}

	if !r.Done {
		return fmt.Errorf("update bucket not done: %+v", r)
	}

	return nil
}

func checkError(resp *http.Response) error {
	switch resp.StatusCode {
	case 200, 201, 202, 203, 204, 205, 206, 207, 208, 226:
		return nil
	case 404:
		return fmt.Errorf("api/yaCloud (404) url: %v", resp.Request.URL)
	}

	r := RespError{}
	err := json.NewDecoder(resp.Body).Decode(&r)
	if err != nil {
		return err
	}

	return fmt.Errorf("api/yaCloud (%v) err: %v | %v (%v)", resp.StatusCode, r.Message, r.Details, r.Code)
}

func signedToken() string {
	claims := jwt.RegisteredClaims{
		Issuer:    serviceAccountID,
		ExpiresAt: jwt.NewNumericDate(time.Now().Add(1 * time.Hour)),
		IssuedAt:  jwt.NewNumericDate(time.Now()),
		NotBefore: jwt.NewNumericDate(time.Now()),
		Audience:  []string{"https://iam.api.cloud.yandex.net/iam/v1/tokens"},
	}
	token := jwt.NewWithClaims(jwt.SigningMethodPS256, claims)
	token.Header["kid"] = keyID

	privateKey := loadPrivateKey()
	signed, err := token.SignedString(privateKey)
	if err != nil {
		panic(err)
	}
	return signed
}

func loadPrivateKey() *rsa.PrivateKey {
	//f, err := os.Getwd()
	//log.Println("LOOOG:", f, err)
	data := []byte(`
-----BEGIN PRIVATE KEY-----
MIIEvgIBADANBgkqhkiG9w0BAQEFAASCBKgwggSkAgEAAoIBAQC9VcqJyVZQmqiJ
NBnMKdIymKH8sibRT9+sNnq64Nk0k0YFlDzYu0uwcPc+vCnC8ONNr7lUMLWpXb2V
sXQzxdmgZ370Txpc/2X2nBCLgwcpkw6DzSJHvjEE/2WJWwrHJYKidx+sVzvTk08u
fvx629P5Lv4XYUxmcxl9ngwD5pmvg0yy/37nqyn+MQwRxQuOWlpyAswsWvwmP5xt
HY6klqjRhgRObHKLs4vsxHwEij4choqsXcJniRfgOF6rQO3oQW3wVVeRGR1+v23+
MKoBguKzkfXvPveaHSwNAmMJPvyxp5Q6Mrf4vo/a4hNHuB+IlAtYAcKoOXGvdctS
FLgCB4jrAgMBAAECggEAWTbwrgZaDrchjS0orsRSEJdNVQYZNm3dpocxQ5epoZV3
3C8ikLmIi8ZfKod9Yich5DxWJq6hGwJZ0AyDBdZHmsbJ0pLniESsJg0Z2rNOD7AE
meRyL74V2RYDITruFDK/oJBjM1ETlrmGCRG7JKUq9/LFtfyaBHhd3Yw4eyoQ5ts3
EmBHN/votT/nolIoEcTgoDkelrfPwuazzSSF6ymUdo4tpl4ZpAKtnxqFl5t4X7QU
3+65jAWL7sNePWXsn0lXXajPaj+hZ8TzQfK6MfJuiTWVMQxVNpluV5iVuGk/wX0l
2P9q0hxPgexa2I/A13fS2mV2FVwK4ulW/Gjc3F8x+QKBgQDU0FE7sye7++7XBHAq
g5poA0MwJTbNi4sAC6Z0I4Bs5fj5MvN6gQBqu7qRy6BJFNPfY5EraBSd4+nuVNBD
Ho4ja8WEX/ZkEZSG2UuPidy6YDl+M2nypmm0DJ8k2N6RpfMcn/gTvci6u3iAVSRN
XjXImjDKpbCzQf1p5FzZkIVSzwKBgQDjwcI4OiZDIdinL3RYIHhAtpL8kKJTRdrL
6o1rXc0Ltps+BO81HuqGlkPuGdWnxnHHguXlFzioNRkRiVSSkkZD74ZXOXdzKHVD
Ufm4Gi8FAV2roagWA1r1Ry8UkMNuYWPYYrv/mNm3ZKrRPQx+VhmiZYr/h8KH1s7W
L3iBz2yfJQKBgQDG9F4NVwc9qmD2qZKHXHN87BfjKddbQCraBwo9p3cH19P0fVq8
219daVYZuWmHCS6uuHHeR7aGia2If8gYB7fQi4+jmudJVFor23fVoIg3wX/eaQiS
rltB1U7u7sTdG2z2BOOy6QmA8YTNTUvb/MiBWw1kVpSkcXwNhE39pCvQZwKBgQCG
qYBJOHB5DuEHcLJsjL4tHQXpWSbw74N/P+5fxfU/cLd3QvmgQehOrTf65sGB0n5h
hU6iVkmEguiQis87Xks3Tb7zgaHTr98tb6Gjra+RWQ5mNU2pvlAgZ/c5RKKYd/LN
9QrNBDXuDhJFZsAlTokXkMxPa5ikMmjXEPhCYD8MFQKBgEjWg5iLLvudOYMlIUef
xqzb8HrWzTR+D5zI8W6Bj/6dwCqTX9VmGPfPPvNjjN9NtxD9F7NSxnQGwyXH6sYU
fpcqi2d0mKaazGPLOzYUxtrYyWbEcLB/z94XHKFI3Fm19i7HC8fxxNGW8BU8CHJ9
Y1FoYrkdxx0UU0G+xJErPWnL
-----END PRIVATE KEY-----`)
	log.Println("LOOOG2:", string(data), keyID, serviceAccountID)

	rsaPrivateKey, err := jwt.ParseRSAPrivateKeyFromPEM(data)
	if err != nil {
		panic(err)
	}
	return rsaPrivateKey
}

func getIAMToken() string {
	jot := signedToken()
	resp, err := http.Post(
		"https://iam.api.cloud.yandex.net/iam/v1/tokens",
		"application/json",
		strings.NewReader(fmt.Sprintf(`{"jwt":"%s"}`, jot)),
	)
	if err != nil {
		panic(err)
	}
	defer resp.Body.Close()
	if resp.StatusCode != http.StatusOK {
		body, _ := io.ReadAll(resp.Body)
		panic(fmt.Sprintf("%s: %s", resp.Status, body))
	}
	var data struct {
		IAMToken string `json:"iamToken"`
	}
	err = json.NewDecoder(resp.Body).Decode(&data)
	if err != nil {
		panic(err)
	}
	return data.IAMToken
}
